searchData={"items":[{"type":"module","title":"Horde.Cluster","doc":"Public functions to join and leave hordes.\n\nCalling `Horde.Cluster.set_members/2` will join the given members in a cluster. Cluster membership is propagated via a CRDT, so setting it once on a single node is sufficient.\n```elixir\n{:ok, sup1} = Horde.DynamicSupervisor.start_link([], name: :supervisor_1, strategy: :one_for_one)\n{:ok, sup2} = Horde.DynamicSupervisor.start_link([], name: :supervisor_2, strategy: :one_for_one)\n{:ok, sup3} = Horde.DynamicSupervisor.start_link([], name: :supervisor_3, strategy: :one_for_one)\n\n:ok = Horde.Cluster.set_members(:supervisor_1, [:supervisor_1, :supervisor_2, :supervisor_3])\n```","ref":"Horde.Cluster.html"},{"type":"function","title":"Horde.Cluster.members/1","doc":"Get the members (nodes) of the horde","ref":"Horde.Cluster.html#members/1"},{"type":"function","title":"Horde.Cluster.set_members/3","doc":"","ref":"Horde.Cluster.html#set_members/3"},{"type":"type","title":"Horde.Cluster.member/0","doc":"","ref":"Horde.Cluster.html#t:member/0"},{"type":"type","title":"Horde.Cluster.name/0","doc":"","ref":"Horde.Cluster.html#t:name/0"},{"type":"behaviour","title":"Horde.DistributionStrategy","doc":"Define your own distribution strategy by implementing this behaviour and configuring Horde to use it.\n\nA few distribution stategies are included in Horde, namely:\n\n- `Horde.UniformDistribution`\n- `Horde.UniformQuorumDistribution`\n- `Horde.UniformRandomDistribution`","ref":"Horde.DistributionStrategy.html"},{"type":"callback","title":"Horde.DistributionStrategy.choose_node/2","doc":"","ref":"Horde.DistributionStrategy.html#c:choose_node/2"},{"type":"callback","title":"Horde.DistributionStrategy.has_quorum?/1","doc":"","ref":"Horde.DistributionStrategy.html#c:has_quorum?/1"},{"type":"type","title":"Horde.DistributionStrategy.member/0","doc":"","ref":"Horde.DistributionStrategy.html#t:member/0"},{"type":"type","title":"Horde.DistributionStrategy.t/0","doc":"","ref":"Horde.DistributionStrategy.html#t:t/0"},{"type":"behaviour","title":"Horde.DynamicSupervisor","doc":"A distributed supervisor.\n\nHorde.DynamicSupervisor implements a distributed DynamicSupervisor backed by a add-wins last-write-wins δ-CRDT (provided by `DeltaCrdt.AWLWWMap`). This CRDT is used for both tracking membership of the cluster and tracking supervised processes.\n\nUsing CRDTs guarantees that the distributed, shared state will eventually converge. It also means that Horde.DynamicSupervisor is eventually-consistent, and is optimized for availability and partition tolerance. This can result in temporary inconsistencies under certain conditions (when cluster membership is changing, for example).\n\nCluster membership is managed with `Horde.Cluster`. Joining a cluster can be done with `Horde.Cluster.set_members/2`. To take a node out of the cluster, call `Horde.Cluster.set_members/2` without that node in the list. Alternatively, setting the `members` startup option to `:auto` will make Horde auto-manage cluster membership so that all (and only) visible nodes are members of the cluster.\n\nEach Horde.DynamicSupervisor node wraps its own local instance of `DynamicSupervisor`. `Horde.DynamicSupervisor.start_child/2` (for example) delegates to the local instance of DynamicSupervisor to actually start and monitor the child. The child spec is also written into the processes CRDT, along with a reference to the node on which it is running. When there is an update to the processes CRDT, Horde makes a comparison and corrects any inconsistencies (for example, if a conflict has been resolved and there is a process that no longer should be running on its node, it will kill that process and remove it from the local supervisor). So while most functions map 1:1 to the equivalent DynamicSupervisor functions, the eventually consistent nature of Horde requires extra behaviour not present in DynamicSupervisor.","ref":"Horde.DynamicSupervisor.html"},{"type":"behaviour","title":"Divergence from standard DynamicSupervisor behaviour - Horde.DynamicSupervisor","doc":"While Horde wraps DynamicSupervisor, it does keep track of processes by the `id` in the child specification. This is a divergence from the behaviour of DynamicSupervisor, which ignores ids altogether. Using DynamicSupervisor is useful for its shutdown behaviour (it shuts down all child processes simultaneously, unlike `Supervisor`).","ref":"Horde.DynamicSupervisor.html#module-divergence-from-standard-dynamicsupervisor-behaviour"},{"type":"behaviour","title":"Graceful shutdown - Horde.DynamicSupervisor","doc":"When a node is stopped (either manually or by calling `:init.stop`), Horde restarts the child processes of the stopped node on another node. The state of child processes is not preserved, they are simply restarted.\n\nTo implement graceful shutdown of worker processes, a few extra steps are necessary.\n\n1. Trap exits. Running `Process.flag(:trap_exit)` in the `init/1` callback of any `worker` processes will convert exit signals to messages and allow running `terminate/2` callbacks. It is also important to include the `shutdown` option in your child spec (the default is 5000ms).\n\n2. Use `:init.stop()` to shut down your node. How you accomplish this is up to you, but by simply calling `:init.stop()` somewhere, graceful shutdown will be triggered.","ref":"Horde.DynamicSupervisor.html#module-graceful-shutdown"},{"type":"behaviour","title":"Module-based Supervisor - Horde.DynamicSupervisor","doc":"Horde supports module-based supervisors to enable dynamic runtime configuration.\n\n```elixir\ndefmodule MySupervisor do\n  use Horde.DynamicSupervisor\n\n  def start_link(init_arg, options \\ []) do\n    Horde.DynamicSupervisor.start_link(__MODULE__, init_arg, options)\n  end\n\n  def init(init_arg) do\n    [strategy: :one_for_one, members: members()]\n    |> Keyword.merge(init_arg)\n    |> Horde.DynamicSupervisor.init()\n  end\n\n  defp members() do\n    []\n  end\nend\n```\n\nThen you can use `MySupervisor.child_spec/1` and `MySupervisor.start_link/1` in the same way as you'd use `Horde.DynamicSupervisor.child_spec/1` and `Horde.DynamicSupervisor.start_link/1`.","ref":"Horde.DynamicSupervisor.html#module-module-based-supervisor"},{"type":"function","title":"Horde.DynamicSupervisor.child_spec/1","doc":"See `start_link/2` for options.","ref":"Horde.DynamicSupervisor.html#child_spec/1"},{"type":"callback","title":"Horde.DynamicSupervisor.child_spec/1","doc":"","ref":"Horde.DynamicSupervisor.html#c:child_spec/1"},{"type":"function","title":"Horde.DynamicSupervisor.count_children/1","doc":"Works like `DynamicSupervisor.count_children/1`.\n\nThis function delegates to all supervisors in the cluster and returns the aggregated output.","ref":"Horde.DynamicSupervisor.html#count_children/1"},{"type":"function","title":"Horde.DynamicSupervisor.init/1","doc":"Works like `DynamicSupervisor.init/1`.","ref":"Horde.DynamicSupervisor.html#init/1"},{"type":"callback","title":"Horde.DynamicSupervisor.init/1","doc":"","ref":"Horde.DynamicSupervisor.html#c:init/1"},{"type":"function","title":"Horde.DynamicSupervisor.start_child/2","doc":"Works like `DynamicSupervisor.start_child/2`.","ref":"Horde.DynamicSupervisor.html#start_child/2"},{"type":"function","title":"Horde.DynamicSupervisor.start_link/1","doc":"Works like `DynamicSupervisor.start_link/1`. Extra options are documented here:\n- `:distribution_strategy`, defaults to `Horde.UniformDistribution`, but more are available - see `Horde.DistributionStrategy`","ref":"Horde.DynamicSupervisor.html#start_link/1"},{"type":"function","title":"Horde.DynamicSupervisor.start_link/3","doc":"","ref":"Horde.DynamicSupervisor.html#start_link/3"},{"type":"function","title":"Horde.DynamicSupervisor.stop/3","doc":"Works like `DynamicSupervisor.stop/3`.","ref":"Horde.DynamicSupervisor.html#stop/3"},{"type":"function","title":"Horde.DynamicSupervisor.terminate_child/2","doc":"Terminate a child process.\n\nWorks like `DynamicSupervisor.terminate_child/2`.","ref":"Horde.DynamicSupervisor.html#terminate_child/2"},{"type":"function","title":"Horde.DynamicSupervisor.wait_for_quorum/2","doc":"Waits for Horde.DynamicSupervisor to have quorum.","ref":"Horde.DynamicSupervisor.html#wait_for_quorum/2"},{"type":"function","title":"Horde.DynamicSupervisor.which_children/1","doc":"Works like `DynamicSupervisor.which_children/1`.\n\nThis function delegates to all supervisors in the cluster and returns the aggregated output. Where memory warnings apply to `DynamicSupervisor.which_children`, these count double for `Horde.DynamicSupervisor.which_children`.","ref":"Horde.DynamicSupervisor.html#which_children/1"},{"type":"type","title":"Horde.DynamicSupervisor.option/0","doc":"","ref":"Horde.DynamicSupervisor.html#t:option/0"},{"type":"type","title":"Horde.DynamicSupervisor.options/0","doc":"","ref":"Horde.DynamicSupervisor.html#t:options/0"},{"type":"module","title":"Horde.DynamicSupervisor.Member","doc":"","ref":"Horde.DynamicSupervisor.Member.html"},{"type":"type","title":"Horde.DynamicSupervisor.Member.status/0","doc":"","ref":"Horde.DynamicSupervisor.Member.html#t:status/0"},{"type":"type","title":"Horde.DynamicSupervisor.Member.t/0","doc":"","ref":"Horde.DynamicSupervisor.Member.html#t:t/0"},{"type":"module","title":"Horde.NodeListener","doc":"A cluster membership manager.\n\nHorde.NodeListener monitors nodes in BEAM's distribution system and\nautomatically adds and removes those marked as `visible` from the cluster it's\nmanaging","ref":"Horde.NodeListener.html"},{"type":"function","title":"Horde.NodeListener.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Horde.NodeListener.html#child_spec/1"},{"type":"function","title":"Horde.NodeListener.init/1","doc":"","ref":"Horde.NodeListener.html#init/1"},{"type":"function","title":"Horde.NodeListener.make_members/1","doc":"","ref":"Horde.NodeListener.html#make_members/1"},{"type":"function","title":"Horde.NodeListener.start_link/1","doc":"","ref":"Horde.NodeListener.html#start_link/1"},{"type":"behaviour","title":"Horde.Registry","doc":"A distributed process registry.\n\nHorde.Registry implements a distributed Registry backed by a δ-CRDT (provided by `DeltaCrdt`). This CRDT is used for both tracking membership of the cluster and implementing the registry functionality itself. Local changes to the registry will automatically be synced to other nodes in the cluster.\n\nCluster membership is managed with `Horde.Cluster`. Joining a cluster can be done with `Horde.Cluster.set_members/2`. To take a node out of the cluster, call `Horde.Cluster.set_members/2` without that node in the list. Alternatively, setting the `members` startup option to `:auto` will make Horde auto-manage cluster membership so that all (and only) visible nodes are members of the cluster.\n\nHorde.Registry supports the common \"via tuple\", described in the [documentation](https://hexdocs.pm/elixir/GenServer.html#module-name-registration) for `GenServer`.\n\nHorde.Registry is API-compatible with `Registry`, with the following exceptions:\n- Horde.Registry does not support `keys: :duplicate`.\n- Horde.Registry does not support partitions.\n- Horde.Registry sends an exit signal to a process when it has lost a naming conflict. See `Horde.Registry.register/3` for details.","ref":"Horde.Registry.html"},{"type":"behaviour","title":"Module-based Registry - Horde.Registry","doc":"Horde supports module-based registries to enable dynamic runtime configuration.\n\n```elixir\ndefmodule MyRegistry do\n  use Horde.Registry\n\n  def start_link(_) do\n    Horde.Registry.start_link(__MODULE__, [keys: :unique], name: __MODULE__)\n  end\n\n  def init(init_arg) do\n    [members: members()]\n    |> Keyword.merge(init_arg)\n    |> Horde.Registry.init()\n  end\n\n  defp members() do\n    [Node.self() | Node.list()]\n    |> Enum.map(fn node -> {__MODULE__, node} end)\n  end\nend\n```\n\nThen you can use `MyRegistry.child_spec/1` and `MyRegistry.start_link/1` in the same way as you'd use `Horde.Registry.child_spec/1` and `Horde.Registry.start_link/1`.","ref":"Horde.Registry.html#module-module-based-registry"},{"type":"function","title":"Horde.Registry.child_spec/1","doc":"See `start_link/2` for options.","ref":"Horde.Registry.html#child_spec/1"},{"type":"callback","title":"Horde.Registry.child_spec/1","doc":"","ref":"Horde.Registry.html#c:child_spec/1"},{"type":"function","title":"Horde.Registry.count/1","doc":"See `Registry.count/1`.","ref":"Horde.Registry.html#count/1"},{"type":"function","title":"Horde.Registry.count_match/4","doc":"See `Registry.count_match/4`.","ref":"Horde.Registry.html#count_match/4"},{"type":"function","title":"Horde.Registry.delete_meta/2","doc":"See `Registry.delete_meta/2`.","ref":"Horde.Registry.html#delete_meta/2"},{"type":"function","title":"Horde.Registry.dispatch/4","doc":"See `Registry.dispatch/4`.","ref":"Horde.Registry.html#dispatch/4"},{"type":"function","title":"Horde.Registry.init/1","doc":"Works like `Registry.init/1`.","ref":"Horde.Registry.html#init/1"},{"type":"callback","title":"Horde.Registry.init/1","doc":"","ref":"Horde.Registry.html#c:init/1"},{"type":"function","title":"Horde.Registry.keys/2","doc":"See `Registry.keys/2`.","ref":"Horde.Registry.html#keys/2"},{"type":"function","title":"Horde.Registry.lookup/1","doc":"See `Registry.lookup/2`.","ref":"Horde.Registry.html#lookup/1"},{"type":"function","title":"Horde.Registry.lookup/2","doc":"","ref":"Horde.Registry.html#lookup/2"},{"type":"function","title":"Horde.Registry.match/4","doc":"See `Registry.match/4`.","ref":"Horde.Registry.html#match/4"},{"type":"function","title":"Horde.Registry.meta/2","doc":"See `Registry.meta/2`.","ref":"Horde.Registry.html#meta/2"},{"type":"function","title":"Horde.Registry.put_meta/3","doc":"See `Registry.put_meta/3`.","ref":"Horde.Registry.html#put_meta/3"},{"type":"function","title":"Horde.Registry.register/3","doc":"Register a process under the given name. See `Registry.register/3`.\n\nWhen 2 clustered registries register the same name at exactly the\nsame time, it will seem like name registration succeeds for both\nregistries. The function returns `{:ok, pid}` for both of these\ncalls.\n\nHowever, due to the eventually consistent nature of the CRDT,\nconflict resolution will take place, and the CRDT will pick one of\nthe two processes as the \"winner\" of the name. The losing process\nwill be sent an exit signal (using `Process.exit/2`) with the\nfollowing reason:\n\n`{:name_conflict, {name, value}, registry_name, winning_pid}`\n\nWhen two registries are joined using `Horde.Cluster.set_members/2`,\nthis name conflict message can also occur.\n\nWhen a cluster is recovering from a netsplit, this name conflict\nmessage can also occur.","ref":"Horde.Registry.html#register/3"},{"type":"function","title":"Horde.Registry.select/2","doc":"See `Registry.select/2`.","ref":"Horde.Registry.html#select/2"},{"type":"function","title":"Horde.Registry.start_link/1","doc":"See `Registry.start_link/1`.\n\nDoes not accept `[partitions: x]`, nor `[keys: :duplicate]` as options.","ref":"Horde.Registry.html#start_link/1"},{"type":"function","title":"Horde.Registry.start_link/3","doc":"","ref":"Horde.Registry.html#start_link/3"},{"type":"function","title":"Horde.Registry.stop/3","doc":"","ref":"Horde.Registry.html#stop/3"},{"type":"function","title":"Horde.Registry.unregister/2","doc":"See `Registry.unregister/2`.","ref":"Horde.Registry.html#unregister/2"},{"type":"function","title":"Horde.Registry.unregister_match/4","doc":"See `Registry.unregister_match/4`.","ref":"Horde.Registry.html#unregister_match/4"},{"type":"function","title":"Horde.Registry.update_value/3","doc":"See `Registry.update_value/3`.","ref":"Horde.Registry.html#update_value/3"},{"type":"type","title":"Horde.Registry.option/0","doc":"","ref":"Horde.Registry.html#t:option/0"},{"type":"type","title":"Horde.Registry.options/0","doc":"","ref":"Horde.Registry.html#t:options/0"},{"type":"module","title":"Horde.UniformDistribution","doc":"Distributes processes to nodes uniformly using a hash ring.\n\nGiven the *same* set of members, it will always start\nthe same process on the same node.","ref":"Horde.UniformDistribution.html"},{"type":"function","title":"Horde.UniformDistribution.choose_node/2","doc":"","ref":"Horde.UniformDistribution.html#choose_node/2"},{"type":"function","title":"Horde.UniformDistribution.has_quorum?/1","doc":"","ref":"Horde.UniformDistribution.html#has_quorum?/1"},{"type":"module","title":"Horde.UniformQuorumDistribution","doc":"Distributes processes to nodes uniformly using a hash ring. Contains a quorum mechanism to handle netsplits.\n\nIt enforces a quorum and will shut down all processes on a node if it is split from the rest of the cluster.","ref":"Horde.UniformQuorumDistribution.html"},{"type":"function","title":"Horde.UniformQuorumDistribution.choose_node/2","doc":"","ref":"Horde.UniformQuorumDistribution.html#choose_node/2"},{"type":"function","title":"Horde.UniformQuorumDistribution.has_quorum?/1","doc":"","ref":"Horde.UniformQuorumDistribution.html#has_quorum?/1"},{"type":"module","title":"Horde.UniformRandomDistribution","doc":"Distributes processes with an uniform probability to\nany node that is alive.","ref":"Horde.UniformRandomDistribution.html"},{"type":"function","title":"Horde.UniformRandomDistribution.choose_node/2","doc":"Selects a random alive node.","ref":"Horde.UniformRandomDistribution.html#choose_node/2"},{"type":"function","title":"Horde.UniformRandomDistribution.has_quorum?/1","doc":"Quorum checks are not enforced, so the process will be\n(re)started on  both sides of a netsplit.","ref":"Horde.UniformRandomDistribution.html#has_quorum?/1"},{"type":"extras","title":"Horde","doc":"# Horde [![Hex pm](http://img.shields.io/hexpm/v/horde.svg?style=flat)](https://hex.pm/packages/horde) [![CircleCI badge](https://circleci.com/gh/derekkraan/horde.svg?style=svg)](https://circleci.com/gh/derekkraan/horde) [![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/horde)\n\n\nDistribute your application over multiple servers with Horde.\n\nHorde is comprised of `Horde.DynamicSupervisor`, a distributed supervisor, and `Horde.Registry`, a distributed registry. Horde is built on top of [DeltaCrdt](https://github.com/derekkraan/delta_crdt_ex).\n\nRead the [full documentation](https://hexdocs.pm/horde) on hexdocs.pm.\n\nThere is an [introductory blog post](https://moosecode.nl/blog/introducing_horde) and a [getting started guide](https://moosecode.nl/blog/getting_started_horde). You can also find me in the Elixir slack channel #horde.\n\nDaniel Azuma gave [a great talk](https://www.youtube.com/watch?v=nLApFANtkHs) at ElixirConf US 2018 where he demonstrated Horde's Supervisor and Registry.\n\nSince Horde is built on CRDTs, it is eventually (as opposed to immediately) consistent, although it does sync its state with its neighbours rather aggressively. Cluster membership in Horde is fully dynamic; nodes can be added and removed at any time and Horde will continue to operate as expected. `Horde.DynamicSupervisor` also uses a hash ring to limit any possible race conditions to times when cluster membership is changing. \n\n`Horde.Registry` is API-compatible with Elixir's own Registry, although it does not yet support the `keys: :duplicate` option. For many use cases, it will be a drop-in replacement. `Horde.DynamicSupervisor` follows the API and behaviour of `DynamicSupervisor` as closely as possible. There will always be some difference between Horde and its standard library equivalents, if not in their APIs, then in their functionality. This is a necessary consequence of Horde's distributed nature.","ref":"readme.html"},{"type":"extras","title":"1.0 release - Horde","doc":"Help us get to 1.0, please fill out our [very short survey](https://docs.google.com/forms/d/e/1FAIpQLSd0fGMuELJIKAiaR1XlvHKjpSo024cojktXjp4ASM7MSXTYfg/viewform?usp=sf_link) and report any issues you encounter when using Horde.","ref":"readme.html#1-0-release"},{"type":"extras","title":"Fault tolerance - Horde","doc":"If a node fails (or otherwise becomes unreachable) then Horde.DynamicSupervisor will redistribute processes among the remaining nodes.\n\nYou can choose what to do in the event of a network partition by specifying `:distribution_strategy` in the options for `Horde.DynamicSupervisor.start_link/2`. Setting this option to `Horde.UniformDistribution` (which is the default) distributes processes using a hash mechanism among all reachable nodes. In the event of a network partition, both sides of the partition will continue to operate. Setting it to `Horde.UniformQuorumDistribution` will operate in the same way, but will shut down if less than half of the cluster is reachable.","ref":"readme.html#fault-tolerance"},{"type":"extras","title":"CAP Theorem - Horde","doc":"Horde is eventually consistent, which means that Horde can guarantee availability and partition tolerancy. Horde cannot guarantee consistency. This means you may end up with duplicate processes in your cluster. Horde does aggressively synchronize between nodes (this is also tunable), but ultimately, depending on the tuning parameters you choose and the quality of the network, there are conditions under which it is possible to have duplicate processes in your cluster. Horde.Registry terminates duplicate processes as soon as they are discovered with a special exit code, so you'll always know when this is happening. See [this page in the docs](https://hexdocs.pm/horde/eventual_consistency.html#horde-registry-merge-conflict) for more details.\n\n_NOTE: Since Horde 0.6.0, Horde.DynamicSupervisor ignores the `id` of a child spec (as Elixir.DynamicSupervisor does), and therefore does not guarantee that each `id` will be unique in the cluster (as it did pre-0.6.0). If you want to uniquely name your processes in a cluster, use Horde.Registry for this purpose. Having both Horde.DynamicSupervisor and Horde.Registry checking for uniqueness was subject to a race condition where Horde.DynamicSupervisor would choose process A to survive and Horde.Registry would choose process B to survive, resulting in both processes being killed._","ref":"readme.html#cap-theorem"},{"type":"extras","title":"Graceful shutdown - Horde","doc":"Using `Horde.DynamicSupervisor.stop/3` will cause the local supervisor to stop and any processes it was running will be shut down and redistributed to remaining supervisors in the horde. (This should happen automatically if `:init.stop()` is called).","ref":"readme.html#graceful-shutdown"},{"type":"extras","title":"Installation - Horde","doc":"Horde is [available in Hex](https://hex.pm/packages/horde).\n\nThe package can be installed by adding `horde` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:horde, \"~> 0.8.5\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Horde","doc":"Here is a small taste of Horde's usage. See the full docs at [https://hexdocs.pm/horde](https://hexdocs.pm/horde) for more information and examples. There is also an example application at `examples/hello_world` that you can refer to if you get stuck.\n\nStarting `Horde.DynamicSupervisor`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n  def start(_type, _args) do\n    children = [\n      {Horde.DynamicSupervisor, [name: MyApp.DistributedSupervisor, strategy: :one_for_one]}\n    ]\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\nAdding a child to the supervisor:\n\n```elixir\n# Add a Task\nHorde.DynamicSupervisor.start_child(MyApp.DistributedSupervisor, %{id: :task, start: {Task, :start_link, [:infinity]}})\n\n# Add an Agent\nHorde.DynamicSupervisor.start_child(MyApp.DistributedSupervisor, %{id: :agent, start: {Agent, :start_link, [fn -> %{} end]}})\n\n# Add a GenServer: You need a previously defined GenServer to call the one\n# liner below.  We have a test (\"graceful shutdown\") in\n# `test/supervisor_test.exs` that exercises and displays that behavior. After\n# defined, it would be very similar to this:\nHorde.DynamicSupervisor.start_child(MyApp.DistributedSupervisor, %{id: :gen_server, start: {GenServer, :start_link, [DefinedGenServer, {500, pid}]}})\n```\n\nAnd so on. The public API should be the same as `Elixir.DynamicSupervisor` (and please open an issue if you find a difference).\n\nJoining supervisors into a single distributed supervisor can be done using `Horde.Cluster`:\n\n```elixir\n{:ok, supervisor_1} = Horde.DynamicSupervisor.start_link(name: :distributed_supervisor_1, strategy: :one_for_one)\n{:ok, supervisor_2} = Horde.DynamicSupervisor.start_link(name: :distributed_supervisor_2, strategy: :one_for_one)\n{:ok, supervisor_3} = Horde.DynamicSupervisor.start_link(name: :distributed_supervisor_3, strategy: :one_for_one)\n\nHorde.Cluster.set_members(:distributed_supervisor_1, [:distributed_supervisor_1, :distributed_supervisor_2, :distributed_supervisor_3])\n# supervisor_1, supervisor_2 and supervisor_3 will be joined in a single cluster.\n```\n\n# Contributing\n\nContributions are welcome! Feel free to open an issue if you'd like to discuss a problem or a possible solution. Pull requests are much appreciated.","ref":"readme.html#usage"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.9.0 - Changelog","doc":"- Bugfixes for scenarios causing Horde to crash. See [#266](https://github.com/derekkraan/horde/pull/266) and [#263](https://github.com/derekkraan/horde/pull/263).\n- [BREAKING] The first parameter of `Horde.DistributionStrategy.choose_node/2` has changed from the identifier to the full child spec. See [#239](https://github.com/derekkraan/horde/pull/239).\n- Use `:erpc` instead of `:rpc`. See [#265](https://github.com/derekkraan/horde/pull/265).\n- Stop eagerly registering processes in Horde.Registry. This solves [#250](https://github.com/derekkraan/horde/issues/250).\n- Add `Horde.UniformRandomDistribution` process distribution strategy. See [#252](https://github.com/derekkraan/horde/pull/252).","ref":"changelog.html#0-9-0"},{"type":"extras","title":"0.8.7 - Changelog","doc":"- Tweak dependency on `:telemetry`","ref":"changelog.html#0-8-7"},{"type":"extras","title":"0.8.6 - Changelog","doc":"- Fix an issue with `members: :auto`","ref":"changelog.html#0-8-6"},{"type":"extras","title":"0.8.5 - Changelog","doc":"- Add support for telemetry version 1.0.0","ref":"changelog.html#0-8-5"},{"type":"extras","title":"0.8.4 - Changelog","doc":"- Upgrade to delta_crdt 0.6.0","ref":"changelog.html#0-8-4"},{"type":"extras","title":"0.8.3 - Changelog","doc":"- Fix a deadlock that occurred if a process was being restarted while `Horde.DynamicSupervisor.start_child/2` was being called. [#218](https://github.com/derekkraan/horde/pull/218)\n- Respect `max_restarts` and `max_seconds` when given as options to `Horde.DynamicSupervisor.child_spec/1`. [#216](https://github.com/derekkraan/horde/pull/216)","ref":"changelog.html#0-8-3"},{"type":"extras","title":"0.8.2 - Changelog","doc":"- Bump version of `telemetry_poller` dependency. [#212](https://github.com/derekkraan/horde/pull/212)","ref":"changelog.html#0-8-2"},{"type":"extras","title":"0.8.1 - Changelog","doc":"- `Horde.Registry.delete_meta/2` has been added to reflect its addition in `Elixir.Registry` in upcoming release 1.11.0 [#208](https://github.com/derekkraan/horde/pull/208)","ref":"changelog.html#0-8-1"},{"type":"extras","title":"0.8.0 - Changelog","doc":"- `Horde.DynamicSupervisor` behaviour in a netsplit has changed. Previously, when a netsplit heals, `Horde.DynamicSupervisor` would try to clean up any duplicate processes. It no longer does this, leaving that responsibility to `Horde.Registry`. [#196](https://github.com/derekkraan/horde/pull/196)\n- `Horde.DynamicSupervisor` and `Horde.Registry` now support the option `members: :auto` to automatically detect other identically-named supervisors or registries. [#184](https://github.com/derekkraan/horde/pull/184)\n- `Horde.DynamicSupervisor` now supports the option `process_redistribution: :active` to rebalance processes actively (aka, when a node joins or leaves the cluster). The default is `:passive`, which only redistributes processes when a node dies or loses quorum. [#164](https://github.com/derekkraan/horde/pull/164).","ref":"changelog.html#0-8-0"},{"type":"extras","title":"0.7.1 - Changelog","doc":"- Use MFA for `on_diff` instead of anonymous function, avoids passing around functions (which can be error-prone). [#167](https://github.com/derekkraan/horde/pull/167).","ref":"changelog.html#0-7-1"},{"type":"extras","title":"0.7.0 - Changelog","doc":"- `Horde.Supervisor` has been renamed to `Horde.DynamicSupervisor`.\n- `Horde.Registry` and `Horde.Supervisor` now follow the api of `Elixir.DynamicSupervisor` more closely (specifically `init/1` callback and module-based Supervisor / Registry). [#152](https://github.com/derekkraan/horde/pull/152).\n- `Horde.Registry.lookup/2` returns `[]` instead of `:undefined` when no match. [#145](https://github.com/derekkraan/horde/pull/145)\n- `child_spec/1` can be overridden in `Horde.Registry` and `Horde.Supervisor` [#135](https://github.com/derekkraan/horde/pull/135) [#143](https://github.com/derekkraan/horde/pull/143)\n- Implement `:listeners` option for Horde.Registry. [#142](https://github.com/derekkraan/horde/pull/142)\n- Fix via tuple usage with meta. [#139](https://github.com/derekkraan/horde/pull/139)","ref":"changelog.html#0-7-0"},{"type":"extras","title":"0.6.1 - Changelog","doc":"- Module-based `Horde.Supervisor` can override `child_spec/1`. [#135](https://github.com/derekkraan/horde/pull/135)\n- Added guides for handling clustering, process state handoff (during deploys), and special considerations for eventual consistency to the [documentation](https://hexdocs.pm/horde).\n- `Horde.Supervisor` now uses libring to distribute processes over nodes. [#130](https://github.com/derekkraan/horde/pull/130)\n- `Horde.Supervisor` publishes metrics with `:telemetry` (`[:horde, :supervisor, :supervised_process_count]`). [#132](https://github.com/derekkraan/horde/pull/132)\n- `Horde.Supervisor` and `Horde.Registry` now support option `delta_crdt_options`, which you can use to tune your cluster. Also updated to the most recent DeltaCRDT. [#100](https://github.com/derekkraan/horde/pull/100)","ref":"changelog.html#0-6-1"},{"type":"extras","title":"0.6.0 - Changelog","doc":"- `Horde.Supervisor` now behaves more like `DynamicSupervisor`. [#122](https://github.com/derekkraan/horde/pull/122)\n- `Horde.Registry` sends an exit signal to the process that \"loses\" when a conflict is resolved. [#118](https://github.com/derekkraan/horde/pull/118)\n- `Horde.Registry.register/3` returns `{:error, {:already_registered, pid}}` when applicable. This improves compatability with `Elixir.Registry`. [#115](https://github.com/derekkraan/horde/pull/115)\n- Adds `Horde.Registry.select/2`, which works the same as `Elixir.Registry.select/2`, which will land in Elixir 1.9. [#110](https://github.com/derekkraan/horde/pull/110)\n- Fixes a bug causing `Horde.Supervisor` to crash if a child process was restarting when `Horde.Supervisor.delete_child/2` was called. [#114](https://github.com/derekkraan/horde/pull/114)","ref":"changelog.html#0-6-0"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThis guide shows you how to get started with Horde. Code samples come from the example HelloWorld app in `examples/hello_world`.","ref":"getting_started.html"},{"type":"extras","title":"Erlang clustering - Getting Started","doc":"Horde relies on Erlang's built in clustering. To make this work, each node in our cluster needs a unique name, and each node must be started using the same cookie. We can try this out locally like so:\n\n```bash\niex --name node1@127.0.0.1 --cookie asdf -S mix\niex --name node2@127.0.0.1 --cookie asdf -S mix\niex --name node3@127.0.0.1 --cookie asdf -S mix\n```\n\nIn this example each node has a unique name, and they all share the same cookie. Now we can connect these nodes by running the following code:\n\n```elixir\nNode.connect(:\"node2@127.0.0.1\")\n```\n\nRun `Node.list()` to confirm that the nodes are connected.\n\nHorde assumes that you will manage Erlang clustering yourself. There are libraries that will help you do this. Continue reading the getting started guide and when you are getting ready to deploy your application, read about [how to set up a cluster](libcluster.html).","ref":"getting_started.html#erlang-clustering"},{"type":"extras","title":"Starting Horde.DynamicSupervisor - Getting Started","doc":"Horde.DynamicSupervisor is API-compatible with Elixir's DynamicSupervisor. There are extra arguments that you can provide, but the basic recipe stays the same:\n\n```elixir\ndefmodule HelloWorld.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Horde.DynamicSupervisor, [name: HelloWorld.HelloSupervisor, strategy: :one_for_one]},\n    ]\n\n    opts = [strategy: :one_for_one, name: HelloWorld.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nThis is also where you can set additional options for Horde.DynamicSupervisor:\n- `:members`, a list of members (if your cluster will have static membership)\n- `:distribution_strategy`, the distribution strategy (`Horde.UniformDistribution` is default)\n- `:delta_crdt_options`, for tuning the delta CRDT that underpins Horde\n\nSee the documentation for `Horde.DynamicSupervisor` for more information.","ref":"getting_started.html#starting-horde-dynamicsupervisor"},{"type":"extras","title":"Starting Horde.Registry - Getting Started","doc":"Horde.DynamicSupervisor is spreading your processes out over the cluster, but how do you know where all these processes are? Horde.Registry is the answer. We want Horde.Registry to be above Horde.DynamicSupervisor in the start-up order.\n\nThis is what our example above looks like with Horde.Registry added in:\n\n```elixir\ndefmodule HelloWorld.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Horde.Registry, [name: HelloWorld.HelloRegistry, keys: :unique]},\n      {Horde.DynamicSupervisor, [name: HelloWorld.HelloSupervisor, strategy: :one_for_one]},\n    ]\n\n    opts = [strategy: :one_for_one, name: HelloWorld.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nThere are also some additional options for Horde.Registry:\n- `:members`, a list of members (if your cluster will have static membership)\n- `:delta_crdt_options`, for tuning the delta CRDT that underpins Horde\n\nSee the documentation for `Horde.Registry` for more information.","ref":"getting_started.html#starting-horde-registry"},{"type":"extras","title":"Running a GenServer - Getting Started","doc":"Let's create a very simple GenServer that we will use to run in our example application to test things out a little.\n\n```elixir\ndefmodule HelloWorld.SayHello do\n  use GenServer\n  require Logger\n\n  def child_spec(opts) do\n    name = Keyword.get(opts, :name, __MODULE__)\n\n    %{\n      id: \"#{__MODULE__}_#{name}\",\n      start: {__MODULE__, :start_link, [name]},\n      shutdown: 10_000,\n      restart: :transient\n    }\n  end\n\n  def start_link(name) do\n    case GenServer.start_link(__MODULE__, [], name: via_tuple(name)) do\n      {:ok, pid} ->\n        {:ok, pid}\n\n      {:error, {:already_started, pid}} ->\n        Logger.info(\"already started at #{inspect(pid)}, returning :ignore\")\n        :ignore\n    end\n  end\n\n  def init(_args) do\n    {:ok, nil}\n  end\n\n  def via_tuple(name), do: {:via, Horde.Registry, {HelloWorld.HelloRegistry, name}}\nend\n```\n\nThis GenServer can be started by running the following line of code: `Horde.DynamicSupervisor.start_child(HelloWorld.HelloSupervisor, HelloWorld.SayHello)`.\n\nOnce running, you can address this GenServer with the following line of code: `GenServer.call(via_tuple(HelloWorld.SayHello), msg)`.","ref":"getting_started.html#running-a-genserver"},{"type":"extras","title":"Next Steps - Getting Started","doc":"Now you should have a working installation of Horde. There is more information available in the other guides, so don't forget to read those too. If you get stuck or have suggestions on how this guide could be improved, please [open an issue](https://github.com/derekkraan/horde/issues/new?title=Improve Getting Started guide) on Github.","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Eventual Consistency","doc":"# Eventual Consistency\n\nHorde uses a CRDT to sync data between nodes. This means that two nodes in your cluster can have a different view of the data, with differences being merged as the nodes sync with each other. We call this \"eventually consistent\", and the result is that we have to deal with merge conflicts and race conditions. Horde's CRDT automatically resolves conflicts, but we still have to deal with the after-effects.","ref":"eventual_consistency.html"},{"type":"extras","title":"Horde.DynamicSupervisor merge conflict - Eventual Consistency","doc":"It is unlikely, but possible, that Horde.DynamicSupervisor will start the same process on two separate nodes.\n\nThis can happen:\n- if using a custom distribution strategy, or\n- when a node dies and not all nodes have the same view of the cluster, or\n- if there is a network partition.\n\nOnce a network partition has healed, Horde will automatically terminate any duplicate processes.","ref":"eventual_consistency.html#horde-dynamicsupervisor-merge-conflict"},{"type":"extras","title":"Horde.Registry merge conflict - Eventual Consistency","doc":"When processes on two different nodes have claimed the same name, this will generate a conflict in Horde.Registry. The CRDT resolves the conflict and Horde.Registry sends an exit signal to the process that lost the conflict. This can be a common occurrence.\n\nUnless this message is handled, it will cause the process to exit. Handling the exit message isn't strictly necessary, because we usually want the process to exit in this case. If for some reason you want to handle the message, simply trap exits in the `init/1` callback and handle the message as follows:\n\n```elixir\ndef init(arg) do\n  Process.flag(:trap_exit, true)\n  {:ok, state}\nend\n\ndef handle_info({:EXIT, _from, {:name_conflict, {key, value}, registry, pid}}, state) do\n  # handle the message, add some logging perhaps, and probably stop the GenServer.\n  {:stop, :normal, state}\nend\n```\n\nNote that, unless your process has `restart: :transient` in its child spec and you have handled the message to shut down the process cleanly, it will be restarted by its supervisor.\n\nUpon restart, it will try to register itself. This will of course fail. If using a via tuple, the following approach is necessary.\n\n```elixir\ndef start_link(arg) do\n  case GenServer.start_link(...) do\n    {:ok, pid} ->\n      {:ok, pid}\n    {:error, {:already_started, pid}} ->\n      :ignore\n  end\nend\n```\n\nIf you are using `Horde.Registry.register/3` in `init/1`, then you must handle `{:error, {:already_registered, pid}}`.\n\n```elixir\ndef init(arg) do\n  case Horde.Registry.register(:my_registry, \"key\", \"value\") do\n    {:ok, _pid} ->\n      {:ok, arg}\n    {:error, {:already_registered, _pid}} ->\n      :ignore\n  end\nend\n```","ref":"eventual_consistency.html#horde-registry-merge-conflict"},{"type":"extras","title":"State Handoff","doc":"# State Handoff\n\nDuring deployment, when a server is restarted, its processes are restarted on other nodes. It can be useful to hand off state from the process that is shutting down to the process that is starting up.\n\nHorde does not offer a specialized state handoff feature, instead we will be using OTP to accomplish state handoff.","ref":"state_handoff.html"},{"type":"extras","title":"Trapping exits - State Handoff","doc":"When `Horde.DynamicSupervisor` is shutting down, it will send an exit signal to its child processes. We need to trap this signal to ensure that our `terminate/2` callback gets called.\n\n```elixir\ndef init(arg) do\n  Process.flag(:trap_exit, true)\n  {:ok, arg}\nend\n```","ref":"state_handoff.html#trapping-exits"},{"type":"extras","title":"Saving state - State Handoff","doc":"We can then save the state in the terminate function.\n\n```elixir\ndef terminate(reason, state) do\n  save_state(state) # save state to Redis, DeltaCRDT, Postgres, Mysql, etc.\nend\n```","ref":"state_handoff.html#saving-state"},{"type":"extras","title":"Shutdown timeout - State Handoff","doc":"The default shutdown timeout is 0, which of course does not leave enough time to save our state. Increase the value of `shutdown` to make sure that your process has enough time to save its data. Be aware: `shutdown` must be set at all levels of your application to apply. A more restrictive shutdown timeout at a higher level in the supervision tree will override less restrictive values.\n\n```elixir\nchildren = [{Horde.DynamicSupervisor, [name: :my_supervisor, shutdown: 1000, strategy: :one_for_one]}]\n```","ref":"state_handoff.html#shutdown-timeout"},{"type":"extras","title":"Restoring state - State Handoff","doc":"Now we have to modify the `init/1` callback to restore the state when the process is initializing. We put this in a `handle_continue/2` callback to avoid making the supervisor wait unnecessarily for the state to load.\n\n```elixir\ndef init(arg) do\n  Process.flag(:trap_exit, true)\n  {:ok, arg, {:continue, :load_state}}\nend\n\ndef handle_continue(:load_state, arg) do\n  {:noreply, load_state(arg)}\nend\n```\n\n### *_!!WARNING!!_*\n\nWe must be careful not to accidentally load invalid state into our processes. For example, if the state of a process changes between deploys, then you might load invalid state and cause your process to crash. Here are some other things to look out for:\n- loading stale data\n- loading data fails, causing process to crash\n- there is no data to load, causing process to crash\n\nAnytime we are loading state into our processes from an external source, we should be very careful. Erlang's \"let it crash\" philosophy is predicated on the idea that a process will be in a known good state after a restart, and if we subvert this by loading invalid state, it could have a negative impact on the stability of our system.","ref":"state_handoff.html#restoring-state"},{"type":"extras","title":"Setting up a Cluster","doc":"# Setting up a Cluster\n\nHorde doesn't provide functionality to set up your cluster, we recommend you use [`libcluster`](https://hexdocs.pm/libcluster/readme.html) for this purpose.\n\nThere are three strategies you can use to integrate libcluster with Horde:","ref":"libcluster.html"},{"type":"extras","title":"Automatic Cluster Membership - Setting up a Cluster","doc":"When starting a `Horde.Registry` or `Horde.DynamicSupervisor`, setting the `members`\noption to have a value of `:auto` will automate membership management. In this\nmode, all visible nodes will be initially added to the cluster. In addition,\nany new nodes that become visible will be automatically added and any\nnodes that shut down will be automatically removed.","ref":"libcluster.html#automatic-cluster-membership"},{"type":"extras","title":"Static Cluster Membership - Setting up a Cluster","doc":"If you will not be adding or removing members from the cluster dynamically, then you can set up libcluster and tell Horde about the members of your cluster. For example, if you run your cluster on bare metal hardware and have a fixed number of servers.\n\n```elixir\nsupervisor_members = [\n  {MyHordeSupervisor, :node1},\n  {MyHordeSupervisor, :node2},\n  {MyHordeSupervisor, :node3},\n  {MyHordeSupervisor, :node4}\n]\n\nregistry_members = [\n  {MyHordeRegistry, :node1},\n  {MyHordeRegistry, :node2},\n  {MyHordeRegistry, :node3},\n  {MyHordeRegistry, :node4}\n]\n\nchildren = [\n  {Horde.Registry, name: MyHordeRegistry, keys: :unique, members: registry_members},\n  {Horde.DynamicSupervisor, name: MyHordeSupervisor, strategy: :one_for_one, members: supervisor_members},\n  ...\n]\n```\n\nThis is the simplest approach. You tell Horde which members are supposed to be in the cluster, and if they are available, Horde will include them in the cluster.","ref":"libcluster.html#static-cluster-membership"},{"type":"extras","title":"Dynamic Cluster Membership - Setting up a Cluster","doc":"If you will be adding and removing nodes from your cluster constantly, and don't want to repackage your application every time you do this, then you will need to perform a couple of extra steps (assuming your needs cannot be met by the [`:auto` setting](#automatic-cluster-membership)).\n\nIn this scenario, you will need to implement a [module-based Supervisor](https://hexdocs.pm/horde/Horde.DynamicSupervisor.html#module-module-based-supervisor)\n\n```elixir\ndefmodule MyHordeSupervisor do\n  use Horde.DynamicSupervisor\n\n  def start_link(_) do\n    Horde.DynamicSupervisor.start_link(\n      __MODULE__,\n      [strategy: :one_for_one],\n      name: __MODULE__\n    )\n  end\n\n  def init(init_arg) do\n    [members: members()]\n    |> Keyword.merge(init_arg)\n    |> Horde.DynamicSupervisor.init()\n  end\n\n  defp members() do\n    [Node.self() | Node.list()]\n    |> Enum.map(fn node -> {__MODULE__, node} end)\n  end\nend\n```\n\nNow every time `MyHordeSupervisor` gets started or restarted, it will compute the members based on the currently connected members.\n\nIn this scenario, you may also want to implement a [module-based Registry](https://hexdocs.pm/horde/Horde.Registry.html#module-module-based-registry)\n\n```elixir\ndefmodule MyHordeRegistry do\n  use Horde.Registry\n\n  def start_link(_) do\n    Horde.Registry.start_link(__MODULE__, [keys: :unique], name: __MODULE__)\n  end\n\n  def init(init_arg) do\n    [members: members()]\n    |> Keyword.merge(init_arg)\n    |> Horde.Registry.init()\n  end\n\n  defp members() do\n    [Node.self() | Node.list()]\n    |> Enum.map(fn node -> {__MODULE__, node} end)\n  end\nend\n```\n\nNow every time `MyHordeRegistry` gets started or restarted, it will compute the members based on the currently connected members.\n\nWe also need a separate process that will listen for `{:nodeup, node}` and `{:nodedown, node}` events and adjust the members of the Horde cluster accordingly. Put this in your supervision tree underneath `MyHordeSupervisor`.\n\n```elixir\ndefmodule NodeListener do\n  use GenServer\n\n  def start_link(_), do: GenServer.start_link(__MODULE__, [])\n\n  def init(_) do\n    :net_kernel.monitor_nodes(true, node_type: :visible)\n    {:ok, nil}\n  end\n\n  def handle_info({:nodeup, _node, _node_type}, state) do\n    set_members(MyHordeRegistry)\n    set_members(MyHordeSupervisor)\n    {:noreply, state}\n  end\n\n  def handle_info({:nodedown, _node, _node_type}, state) do\n    set_members(MyHordeRegistry)\n    set_members(MyHordeSupervisor)\n    {:noreply, state}\n  end\n\n  defp set_members(name) do\n    members =\n      [Node.self() | Node.list()]\n      |> Enum.map(fn node -> {name, node} end)\n\n    :ok = Horde.Cluster.set_members(name, members)\n  end\nend\n```\n\nNote that the funcionality provided in this example is essentially the same as\nthe [`members: :auto` setting](#automatic-cluster-membership), however setting\nit up yourself allows greater flexability to modify it if `:auto` mode doesn't\nmeet your requirements.","ref":"libcluster.html#dynamic-cluster-membership"}],"content_type":"text/markdown"}